;==================================
; program for FLAG PMI
; robot ISIA 2001 - Space Odyssee
; www-isia.cma.fr/~robotm6/
; target = at90s1200, 8MHz
; compiled with avr studio from Atmel
;==================================

.include "1200def.inc"

;=====================
; Parametres reglables
;=====================

;duree au bout de laquelle on monte le drapeau
.equ	TIME_VENTOUSAGE 	= 2  ; secondes
.equ	TIMEOUT_VENTOUSAGE 	= 3  ; secondes
.equ	TIME_MONTE_DRAPEAU 	= 10 ; secondes
.equ	TIMEOUT_MONTE_DRAPEAU 	= 10 ; secondes
.equ	HAUTEUR_MONTE_DRAPEAU 	= 2 ; graduations si enable
.equ	TIME_CLIGNOTE 		= 240 ;

;=====================
; Port mapping
;=====================
; Input 7 bits:
.equ	PORT_INPUT	= PORTD
.equ	PIN_INPUT	= PIND
.equ	DIR_INPUT	= DDRD
.equ	ILS_DEPART	= 0x00	
.equ	DETECT_ALTITUDE	= 0x04
.equ	REGLE_ALTI_0    = 0x03
; pas utiliss :
.equ	DETECT_SYNCRO	= 0x02 ; interup
.equ	INTER_POSE 	= 0x00 ; no
.equ	DETECT_COLOR	= 0x06
.equ	REGLE_ALTI_1    = 0x05

; output 7 bits:
.equ	PORT_OUTPUT	= PORTB
.equ	DIR_OUTPUT	= DDRB
;.equ			= 0x01
;.equ			= 0x02
.equ	MOT_VENTOUSE_ON	= 0x04
.equ	MOT_DRAPEAU_ON	= 0x08
;.equ			= 0x10	
.equ	LED_VENTOUSAGE	= 0x20
.equ	LED_STATE	= 0x40	
.equ	LED_DRAPEAU	= 0x80	

;===========================
; TIMER, constante pour que 
; CK*1024*(255-timeD) = 0.039s
; pas de temps du compteur = 0.039s
; => octet de poids fort en seconde !
;===========================
.equ 	timeD		= 224
	
;========================
; variables
;========================
; Counters
.def	counter0 	= r23
.def	counter1 	= r24
.def	delaycounterL 	= r21
.def	delaycounterH 	= r22

.def  	timeFromBeginingL = r19
.def	timeFromBeginingH = r20

.def	outputreg 	= r25
.def 	altitude 	= r26

.def	TmpTimer	=r16
.def	Tmp		=r18
.def	Tmp2		=r17

;===========================
; 	EEPROM
;===========================
.eseg					;EEPROM segment
.org 0

;===========================
;	SOURCE CODE
;===========================	
.cseg					;CODE segment
.org $000
	rjmp __reset_
.org INT0addr
	rjmp __ext_int0_
.org OVF0addr
	rjmp __tim0_ovf_
.org ACIaddr
	reti
	
;========================
; program setup
;========================	
__reset_:
;definir la direction des ports
	ldi Tmp, 0xff
	out DIR_OUTPUT,Tmp
	ldi Tmp, 0x00
	out DIR_INPUT, Tmp
; set pull up for input
	ldi Tmp, 0xff
	out PORT_INPUT,Tmp
;couper tous les moteurs
	clr outputreg
	out PORT_OUTPUT, outputreg
;initialiser interruptions : disable all
	clr Tmp
	out GIMSK, Tmp ; external int
	clr Tmp
	out TIMSK, Tmp ; timer int
;disable interupts
	cli
;fin, commence le code
	rjmp start

;=========================
; interruption externe
;=========================
__ext_int0_ : 
	; tue l'interruption
	clr Tmp
	out GIMSK, Tmp
	reti

;=========================
; interruption timer
;=========================
__tim0_ovf_ :
	; met a jour l'heure
time_plus_second :
	inc	timeFromBeginingL
	brne	endtimeplus
	inc	timeFromBeginingH
endtimeplus:
	; reinitialise le compteur  la bonne valeur
	clr 	TmpTimer
	out 	TIFR, TmpTimer
	ldi	TmpTimer, timeD
	out 	TCNT0, TmpTimer
	reti
	
;=========================
;	delay
;=========================	
;delay : 
	mov	counter1, delaycounterH
delay1:	mov	counter0, delaycounterL
delay2:	dec	counter0
	brne	delay2
	dec	counter1
	brne	delay1
	ret	
	
;=========================
; mode idle
;=========================
start_idle_mode :
	;timer interupt disable
	ldi 	Tmp, 0x00
	out 	TIMSK, Tmp
	;external interrupt rising
	ldi 	Tmp, 0x23
	;external interrupt falling
	;ldi 	Tmp,0x22
	;external interrupt lowlevel
	;ldi 	Tmp,0x20
	out 	MCUCR, Tmp
	;external interrupt enable
	ldi 	Tmp, 0x40
	out 	GIMSK, Tmp
	sei
	sleep	; passe en idle mode, attend interruption externe
	rjmp	idle_mode_callback ; 
	; suite dans __ext_int0_
loop1:
	rjmp loop1

;===============================
; initialisation de l'horologe
;===============================	
time_reset :
	clr	timeFromBeginingL
	clr	timeFromBeginingH
	; set period to 1 second
	ldi 	TmpTimer,0x05	; CK/1024
	out 	TCCR0, TmpTimer
	; initialise le compteur  la bonne valeur
	ldi	TmpTimer, timeD
	out 	TCNT0, TmpTimer
	clr 	TmpTimer
	out 	TIFR, TmpTimer
	;enable timer interrupt
	ldi 	TmpTimer,0x02
	out 	TIMSK, TmpTimer
	sei
	rjmp	timer_init_callback	
;=========================
; PROGRAM_PRINCIPAL
;=========================
start:	
	clr 	outputreg ; coupes les sorties
	out 	PORT_OUTPUT,outputreg
	ldi	Tmp2,TIME_CLIGNOTE
	
wait_in :
idle_mode_callback:
	sbis 	PIN_INPUT, ILS_DEPART
	rjmp	wait_in
	
;----- Attend d'etre largue -------
wait_largue:
	cbr 	outputreg, LED_STATE
	out 	PORT_OUTPUT,outputreg
	sbic	PIN_INPUT, ILS_DEPART
	rjmp 	wait_largue
	
;------ on vient de se faire larguer ------
	rjmp 	time_reset
timer_init_callback:	
	sbr 	outputreg, LED_STATE 
	out 	PORT_OUTPUT,outputreg
	ldi 	Tmp, TIME_VENTOUSAGE
	add 	Tmp, timeFromBeginingH
wait_ventousage:
	cp 	timeFromBeginingL, Tmp2 
	brsh 	led_on1
led_off1:
	cbr	outputreg, LED_STATE
     	out 	PORT_OUTPUT,outputreg
        rjmp 	wait_ventousage_suite
led_on1:
	sbr	outputreg, LED_STATE
     	out 	PORT_OUTPUT,outputreg
        rjmp 	wait_ventousage_suite
wait_ventousage_suite :      
	cp 	timeFromBeginingH, Tmp 
	brsh 	ventousage
	sbic 	PIN_INPUT, ILS_DEPART 
	rjmp 	wait_largue ; on s'est gour, on n'a pas t largu
        rjmp 	wait_ventousage
	
;------- on se ventouse ---------
ventousage : 
; lire d'abord la hauteur a laquelle on doit monter
	ldi	altitude, HAUTEUR_MONTE_DRAPEAU
	sbis 	PIN_INPUT, DETECT_ALTITUDE
	ldi	altitude,0x00
     	sbr	outputreg, LED_VENTOUSAGE
     	sbr	outputreg, MOT_VENTOUSE_ON
     	out 	PORT_OUTPUT,outputreg
     	ldi 	Tmp, TIMEOUT_VENTOUSAGE
	add 	Tmp, timeFromBeginingH
wait_timeout_ventousage:
	cp 	timeFromBeginingH, Tmp 
	brsh 	wait_time_flag
        rjmp 	wait_timeout_ventousage
        
        
;---- on attends l'heure de monter le drapeau
wait_time_flag:
	cbr	outputreg, LED_VENTOUSAGE
     	cbr	outputreg, MOT_VENTOUSE_ON
     	out 	PORT_OUTPUT,outputreg
     	ldi 	Tmp, TIME_MONTE_DRAPEAU
	add 	Tmp, timeFromBeginingH
wait_wait_time_flag:
	cp 	timeFromBeginingL, Tmp2 
	brsh 	led_on2
led_off2:
	cbr	outputreg, LED_STATE
     	out 	PORT_OUTPUT,outputreg
        rjmp 	wait_wait_time_flag_suite
led_on2:
	sbr	outputreg, LED_STATE
     	out 	PORT_OUTPUT,outputreg
        rjmp 	wait_wait_time_flag_suite
wait_wait_time_flag_suite :     
	cp 	timeFromBeginingH, Tmp 
	brsh 	monte_flag
fin_wait_wait_time_flag :      
        rjmp 	wait_wait_time_flag
      
                
monte_flag:
 	; allume led drapeau
 	sbr outputreg, LED_STATE ; set bit 
	sbr outputreg, LED_DRAPEAU ; set bit 
	out PORT_OUTPUT,outputreg
		
	sbr 	outputreg, MOT_DRAPEAU_ON ; set bit 
	out 	PORT_OUTPUT,outputreg
	clr 	Tmp
; regarde si on doit monter le drapeau
	cp 	Tmp, altitude 
	brsh 	stop_drapeau
	
	ldi	delaycounterH, TIMEOUT_MONTE_DRAPEAU 
	add	delaycounterH, timeFromBeginingH
wait_drapeau_haut:
	cbr 	outputreg, LED_VENTOUSAGE ; clear bit 
	out 	PORT_OUTPUT,outputreg
	sbic 	PIN_INPUT, DETECT_ALTITUDE
	rjmp	metre_noir
	cp 	timeFromBeginingH, delaycounterH 
	brsh 	stop_drapeau
	rjmp	wait_drapeau_haut
	
metre_noir:
	sbr 	outputreg, LED_VENTOUSAGE ; clear bit 
	out 	PORT_OUTPUT,outputreg
	inc	Tmp
	cp 	Tmp, altitude 
	brsh 	stop_drapeau
	
wait_metre_blanc:
	sbis 	PIN_INPUT, DETECT_ALTITUDE
	rjmp	wait_drapeau_haut
	cp 	timeFromBeginingH, delaycounterH 
	brsh 	stop_drapeau
	rjmp	wait_metre_blanc
	
stop_drapeau:
	cbr 	outputreg, MOT_DRAPEAU_ON 
	cbr 	outputreg, LED_DRAPEAU
	out 	PORT_OUTPUT,outputreg
	rjmp 	fin
	
	;---- on stop le cpu -----
fin :
	; etteind led state
	cbr 	outputreg, LED_STATE ; clear bit 
	cbr 	outputreg, LED_DRAPEAU ; clear bit 
	cbr 	outputreg, LED_VENTOUSAGE ; clear bit 
	cbr 	outputreg, MOT_DRAPEAU_ON ; clear bit 
	cbr 	outputreg, MOT_VENTOUSE_ON ; clear bit 
	out 	PORT_OUTPUT,outputreg
start_powerdown_mode :
	;interupts disable
	cli
	;external interrupt rising & powerdown mode
	ldi 	Tmp, 0x33
	out 	MCUCR, Tmp
	sleep	; passe en power down mode, attend reset
	rjmp 	fin
