;==================================
; program for FLAG PMI
; robot ISIA 2001 - Space Odyssee
; www-isia.cma.fr/~robotm6/
; target = at90s1200, 8MHz
; compiled with avr studio from Atmel
;==================================

.include "1200def.inc"

;=====================
; Parametres reglbles
;=====================
; choix de la strategie
.equ 	BOOL_POUSSE_PLANETE	= 1
.equ	BOOL_TIRE_PLANETE	= 1
.equ	BOOL_MONTE_DRAPEAU	= 1

; duree des taches (TIME)
; ou duree des delais d'attentes en cas de probleme (TIMEOUT)
.equ	TIMEOUT_AVANCE_1 	= 5 ; secondes
.equ	TIMEOUT_VELCROTISE 	= 5  ; secondes 
.equ	TIMEOUT_SOULEVE		= 10 ; secondes 
.equ	TIME_AVANCE_2 		= 10 ; secondes
.equ	TIME_WAIT_1		= 10 ; secondes ***
.equ	TIME_RECULE 		= 15 ; secondes
.equ	TIMEOUT_MONTE 		= 15 ; secondes
;duree au bout de laquelle on monte le drapeau
.equ	TIME_MONTE_DRAPEAU 	= 60 ; secondes ***
.equ	TIMEOUT_MONTE_DRAPEAU	= 15 ; secondes


;=====================
; Port mapping
;=====================
; Input 7 bits:
.equ	PORT_INPUT	= PORTD
.equ	PIN_INPUT	= PIND
.equ	DIR_INPUT	= DDRD
.equ	DETECT_SYNCRO	= 0x02 ; interup
.equ	ILS_DEPART	= 0x00	
.equ	INTER_COLLISION = 0x01
.equ	INTER_BIEN_POSE = 0x03
.equ	INTER_HAUT	= 0x04
.equ	INTER_BAS	 = 0x05
.equ	DETECT_DRAPEAU_OUT =0x06

; output 8 bits:
.equ	PORT_OUTPUT	= PORTB
.equ	DIR_OUTPUT	= DDRB
.equ	MOT_AVANCE_DIR	= 0x01
.equ	MOT_MONTE_DIR	= 0x02	
.equ	MOT_AVANCE_ON	= 0x04
.equ	MOT_MONTE_ON	= 0x08	; plaque descend =>dir=1
.equ	MOT_DRAPEAU_ON	= 0x10	
;.equ			= 0x20
.equ	LED_STATE	= 0x40	
.equ	LED_DRAPEAU	= 0x80	


;===========================
; TIMER, constante pour que 
; CK*1024*(255-timeD) = 0.039s
; pas de temps du compteur = 0.039s
; => octet de poids fort en seconde !
;===========================
.equ 	timeD		= 224
	
;========================
; variables
;========================
; Counters
.def	counter0 	= r23
.def	counter1 	= r24
.def	delaycounterL 	= r21
.def	delaycounterH 	= r22

.def  	timeFromBeginingL = r19
.def	timeFromBeginingH = r20

.def	outputreg 	= r25
.def 	altitude 	= r26

.def	TmpTimer	=r16
.def	TmpB		=r17
.def	Tmp		=r18

;===========================
; 	EEPROM
;===========================
.eseg					;EEPROM segment
.org 0

;===========================
;	SOURCE CODE
;===========================	
.cseg					;CODE segment
.org $000
	rjmp __reset_
.org INT0addr
	rjmp __ext_int0_
.org OVF0addr
	rjmp __tim0_ovf_
.org ACIaddr
	reti
	
;========================
; program setup
;========================	
__reset_:
;definir la direction des ports
	ldi Tmp, 0xff
	out DIR_OUTPUT,Tmp
	ldi Tmp, 0x00
	out DIR_INPUT, Tmp
; set pull up for input
	ldi Tmp, 0xff
	out PORT_INPUT,Tmp
;couper tous les moteurs
	clr outputreg
	out PORT_OUTPUT, outputreg
;initialiser interruptions : disable all
	ldi Tmp,0x00
	out GIMSK, Tmp ; external int
	ldi Tmp,0x00
	out TIMSK, Tmp ; timer int
;disable interupts
	cli
;fin, commence le code
	rjmp start

;=========================
; interruption externe
;=========================
__ext_int0_ : 
	; tue l'interruption
	ldi Tmp,0x00
	out GIMSK, Tmp
	; c'est parti mon kiki !!!
	reti

;=========================
; interruption timer
;=========================
__tim0_ovf_ :
	; met a jour l'heure
time_plus_second :
	inc	timeFromBeginingL
	brne	endtimeplus
	inc	timeFromBeginingH
endtimeplus:
	; reinitialise le compteur  la bonne valeur
	clr 	TmpTimer
	out 	TIFR, TmpTimer
	ldi	TmpTimer, timeD
	out 	TCNT0, TmpTimer
	reti
	
;=========================
;	delay
;=========================	
;delay : 
	mov	counter1, delaycounterH
delay1:	mov	counter0, delaycounterL
delay2:	dec	counter0
	brne	delay2
	dec	counter1
	brne	delay1
	ret	
	
;===============================
; initialisation de l'horologe
;===============================	
time_reset :
	ldi	timeFromBeginingL, 0x00
	ldi	timeFromBeginingH, 0x00
	; set period to 1 second
	ldi 	TmpTimer,0x05	; CK/1024
	out 	TCCR0, TmpTimer
	; initialise le compteur  la bonne valeur
	ldi	TmpTimer, timeD
	out 	TCNT0, TmpTimer
	clr 	TmpTimer
	out 	TIFR, TmpTimer
	;enable timer interrupt
	ldi 	TmpTimer,0x02
	out 	TIMSK, TmpTimer
	sei
	rjmp	delayy	
;=========================
; PROGRAM_PRINCIPAL
;=========================
start:	
	; on demarre, on economise les batteries
	; passer en mode sleep et attendre le dpart du match
	clr 	outputreg
	out 	PORT_OUTPUT,outputreg
	rjmp 	time_reset

delayy:	
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,TIMEOUT_AVANCE_1
	add 	Tmp,timeFromBeginingH
wait_collision3:
	cp 	timeFromBeginingH, Tmp
	brsh 	start_motor1
	rjmp 	wait_collision3	
	
	;----- on Demarre ------
	; on va vers l'avant
start_motor1:
	sbr 	outputreg, MOT_AVANCE_DIR ; set bit
	out 	PORT_OUTPUT,outputreg
	sbr 	outputreg, MOT_AVANCE_ON ; set bit 
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,TIMEOUT_AVANCE_1
	add 	Tmp,timeFromBeginingH
wait_collision:
	cp 	timeFromBeginingH, Tmp
	brsh 	stop_motor1
	rjmp 	wait_collision	
	
stop_motor1:
	cbr 	outputreg, MOT_AVANCE_ON ; set bit 
	out 	PORT_OUTPUT,outputreg	
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,TIMEOUT_AVANCE_1
	add 	Tmp,timeFromBeginingH
wait_stopmotor1:
	cp 	timeFromBeginingH, Tmp
	brsh 	start_motor2
	rjmp 	wait_stopmotor1	
	
	
start_motor2:
	sbr 	outputreg, MOT_MONTE_DIR ; set bit
	out 	PORT_OUTPUT,outputreg
	sbr 	outputreg, MOT_MONTE_ON ; set bit 
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,TIMEOUT_AVANCE_1
	add 	Tmp,timeFromBeginingH
wait_collision2:
	cp 	timeFromBeginingH, Tmp
	brsh 	stop_motor2
	rjmp 	wait_collision2	
	
stop_motor2:
	cbr 	outputreg, MOT_MONTE_ON ; set bit 
	out 	PORT_OUTPUT,outputreg	
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,TIMEOUT_AVANCE_1
	add 	Tmp,timeFromBeginingH
wait_stopmotor2:
	cp 	timeFromBeginingH, Tmp
	brsh 	fin
	rjmp 	wait_stopmotor2	
	
fin :	
	; stop moteurs
	cbr 	outputreg, MOT_AVANCE_ON ; set bit
	cbr 	outputreg, MOT_MONTE_ON ; set bit
	cbr 	outputreg, MOT_DRAPEAU_ON ; set bit 
	; etteind led state
	cbr 	outputreg, LED_STATE ; clear bit 
	out 	PORT_OUTPUT,outputreg	
start_powerdown_mode :
	;interupts disable
	cli
	;external interrupt rising & powerdown mode
	ldi 	Tmp, 0x33
	out 	MCUCR, Tmp
	sleep	; passe en powerdown mode, attend reset
	rjmp fin

