;==================================
; program for FLAG PMI
; robot ISIA 2001 - Space Odyssee
; www-isia.cma.fr/~robotm6/
; target = at90s1200, 8MHz
; compiled with avr studio from Atmel
;==================================

.include "1200def.inc"

;=====================
; Parametres reglables
;=====================

;duree au bout de laquelle on monte le drapeau
.equ	TIME_VENTOUSAGE 	= 5  ; secondes
.equ	TIME_MONTE_DRAPEAU 	= 60 ; secondes
.equ	TIMEOUT_MONTE_DRAPEAU 	= 20 ; secondes


;=====================
; Port mapping
;=====================
; Input 7 bits:
.equ	PORT_INPUT	= PORTD
.equ	PIN_INPUT	= PIND
.equ	DIR_INPUT	= DDRD
.equ	DETECT_SYNCRO	= 0x02 ; interup
.equ	ILS_DEPART	= 0x01	
.equ	INTER_POSE 	= 0x00
.equ	DETECT_COLOR	= 0x03
.equ	DETECT_ALTITUDE	= 0x04
.equ	REGLE_ALTI_0    = 0x05
.equ	REGLE_ALTI_1    = 0x06

; output 7 bits:
.equ	PORT_OUTPUT	= PORTB
.equ	DIR_OUTPUT	= DDRB
.equ	MOT_VENOUSE_ON	= 0x01
.equ	MOT_DRAPEAU_ON	= 0x02
;.equ			= 0x04
;.equ			= 0x08
;.equ			= 0x10	
.equ	LED_IDLE_STOP	= 0x20
.equ	LED_STATE	= 0x40	
.equ	LED_DRAPEAU	= 0x80	

;===========================
; TIMER, constante pour que 
; CK*1024*(255-timeD) = 0.039s
; pas de temps du compteur = 0.039s
; => octet de poids fort en seconde !
;===========================
.equ 	timeD		= 224
	
;========================
; variables
;========================
; Counters
.def	counter0 	= r23
.def	counter1 	= r24
.def	delaycounterL 	= r21
.def	delaycounterH 	= r22

.def  	timeFromBeginingL = r19
.def	timeFromBeginingH = r20

.def	outputreg 	= r25
.def 	altitude 	= r26

.def	TmpTimer	=r16
.def	TmpB		=r17
.def	Tmp		=r18

;===========================
; 	EEPROM
;===========================
.eseg					;EEPROM segment
.org 0

;===========================
;	SOURCE CODE
;===========================	
.cseg					;CODE segment
.org $000
	rjmp __reset_
.org INT0addr
	rjmp __ext_int0_
.org OVF0addr
	rjmp __tim0_ovf_
.org ACIaddr
	reti
	
;========================
; program setup
;========================	
__reset_:
;definir la direction des ports
	ldi Tmp, 0xff
	out DIR_OUTPUT,Tmp
	ldi Tmp, 0x00
	out DIR_INPUT, Tmp
; set pull up for input
	ldi Tmp, 0xff
	out PORT_INPUT,Tmp
;couper tous les moteurs
	clr outputreg
	out PORT_OUTPUT, outputreg
;initialiser interruptions : disable all
	clr Tmp
	out GIMSK, Tmp ; external int
	clr Tmp
	out TIMSK, Tmp ; timer int
;disable interupts
	cli
;fin, commence le code
	rjmp start

;=========================
; interruption externe
;=========================
__ext_int0_ : 
	; tue l'interruption
	clr Tmp
	out GIMSK, Tmp
	reti

;=========================
; interruption timer
;=========================
__tim0_ovf_ :
	; met a jour l'heure
time_plus_second :
	inc	timeFromBeginingL
	brne	endtimeplus
	inc	timeFromBeginingH
endtimeplus:
	; reinitialise le compteur  la bonne valeur
	clr 	TmpTimer
	out 	TIFR, TmpTimer
	ldi	TmpTimer, timeD
	out 	TCNT0, TmpTimer
	reti
	
;=========================
;	delay
;=========================	
;delay : 
	mov	counter1, delaycounterH
delay1:	mov	counter0, delaycounterL
delay2:	dec	counter0
	brne	delay2
	dec	counter1
	brne	delay1
	ret	
	
;=========================
; mode idle
;=========================
start_idle_mode :
	;timer interupt disable
	ldi 	Tmp, 0x00
	out 	TIMSK, Tmp
	;external interrupt rising
	ldi 	Tmp, 0x23
	;external interrupt falling
	;ldi 	Tmp,0x22
	;external interrupt lowlevel
	;ldi 	Tmp,0x20
	out 	MCUCR, Tmp
	;external interrupt enable
	ldi 	Tmp, 0x40
	out 	GIMSK, Tmp
	sei
	sleep	; passe en idle mode, attend interruption externe
	rjmp	match_start ; 
	; suite dans __ext_int0_
loop1:
	rjmp loop1

;===============================
; initialisation de l'horologe
;===============================	
time_reset :
	clr	timeFromBeginingL
	clr	timeFromBeginingH
	; set period to 1 second
	ldi 	TmpTimer,0x05	; CK/1024
	out 	TCCR0, TmpTimer
	; initialise le compteur  la bonne valeur
	ldi	TmpTimer, timeD
	out 	TCNT0, TmpTimer
	clr 	TmpTimer
	out 	TIFR, TmpTimer
	;enable timer interrupt
	ldi 	TmpTimer,0x02
	out 	TIMSK, TmpTimer
	sei
	rjmp	wait_largue	
;=========================
; PROGRAM_PRINCIPAL
;=========================
start:	
	; on demarre, on economise les batteries
	; passer en mode sleep et attendre le dpart du match
	rjmp start_idle_mode
	
match_start :
	sbr 	outputreg, LED_IDLE_STOP ; set bit led 1
	out 	PORT_OUTPUT,outputreg
        ; check if it is really the begining, we must be under the ils
        sbis	PIN_INPUT, ILS_DEPART
        rjmp 	start
        
real_match_start:
	; disable internal interrupt
	clr Tmp
	out GIMSK, Tmp ; external int
	; LED 0 (start) ON
	sbr 	outputreg, LED_STATE ; set bit led 1
	out 	PORT_OUTPUT,outputreg
	
	;----- read altitude requested -----
	ldi	altitude, 0x00
	ldi	Tmp, 0x02
	sbic 	PIN_INPUT, REGLE_ALTI_0
	inc	altitude
	sbic 	PIN_INPUT, REGLE_ALTI_1
	add	altitude,Tmp
	
	;----- timer reset -------
	rjmp 	time_reset
	
	;----- Attend d'etre largue -------
wait_largue:
	sbic	PIN_INPUT, ILS_DEPART
	rjmp 	wait_largue

	;----- Attend de toucher le sol et bonne couleur-----
wait_pose:
	sbic	PIN_INPUT, INTER_POSE
	rjmp 	wait_pose
	sbic	PIN_INPUT, DETECT_COLOR  ;********* REALLY ?????? *******
	rjmp 	wait_pose		  ;********* REALLY ?????? *******
	
		
	;----- ventousage ------
	; on demare le moteur de la ventouse, il s'arrete tout seul
ventousage :
	sbr outputreg, MOT_VENOUSE_ON ; set bit
	out PORT_OUTPUT,outputreg
	
	;----- attend un peu -----
	ldi Tmp, TIME_VENTOUSAGE
	add Tmp, timeFromBeginingH
wait_ventousage:
	cp timeFromBeginingH, Tmp 
	brsh wait_the_good_time
	rjmp wait_ventousage
	
	;---- on attend que ce soit l'heure de monter le drapeau------
wait_the_good_time:
	ldi Tmp, TIME_MONTE_DRAPEAU
wait_drapeau:
	cp timeFromBeginingH, Tmp 
	brsh its_time_monte_drapeau
	rjmp wait_drapeau
	
	;---- c'est l'heure de monter le drapeau -----
its_time_monte_drapeau :
 	; allume led drapeau
	sbr outputreg, LED_DRAPEAU ; set bit 
	out PORT_OUTPUT,outputreg
		
	; verifie encore qu'on est bien pose
wait_bienpose:
	sbic	PIN_INPUT, INTER_POSE
	rjmp 	wait_bienpose
	sbic	PIN_INPUT, DETECT_COLOR  ;********* REALLY ?????? *******
	rjmp 	wait_bienpose		  ;********* REALLY ?????? *******
	
	;---- monte drapeau -----
	cpi	altitude, 0x00	;if equal then don't lift up the flag
	breq	fin
monte_drapeau:
	sbr 	outputreg, MOT_DRAPEAU_ON ; set bit 
	out 	PORT_OUTPUT,outputreg
	clr 	Tmp
	ldi	delaycounterH, TIMEOUT_MONTE_DRAPEAU 
	add	delaycounterH, timeFromBeginingH
wait_drapeau_haut:
	sbic 	PIN_INPUT, DETECT_ALTITUDE
	rjmp	metre_noir
	cp 	timeFromBeginingH, delaycounterH 
	brsh 	stop_drapeau
	rjmp	wait_drapeau_haut
	
metre_noir:
	inc	Tmp
	cp 	Tmp, altitude 
	brsh 	stop_drapeau
	
wait_metre_blanc:
	sbis 	PIN_INPUT, DETECT_ALTITUDE
	rjmp	wait_drapeau_haut
	cp 	timeFromBeginingH, delaycounterH 
	brsh 	stop_drapeau
	rjmp	wait_metre_blanc
	
stop_drapeau:
	cbr 	outputreg, MOT_DRAPEAU_ON ; set bit 
	out 	PORT_OUTPUT,outputreg
	rjmp 	fin
	
	;---- on stop le cpu -----
fin :
	; etteind led state
	cbr 	outputreg, LED_STATE ; clear bit 
	out 	PORT_OUTPUT,outputreg
start_powerdown_mode :
	;interupts disable
	cli
	;external interrupt rising & powerdown mode
	ldi 	Tmp, 0x33
	out 	MCUCR, Tmp
	sleep	; passe en power down mode, attend reset
	rjmp 	fin
