;==================================
; program for Velcro PMI
; robot ISIA 2001 - Space Odyssee
; www-isia.cma.fr/~robotm6/
; target = at90s1200, 8MHz
; compiled with avr studio from Atmel
;==================================

.include "1200def.inc"

;=====================
; Parametres reglbles
;=====================
; duree pendant laquelle on avance quand on a velcro tis la planete
.equ	MAX_TIME_AVANCE_L = 1
.equ	MAX_TIME_AVANCE_H = 2

.equ	DUREE_POUR_VELCROTISER_L = 255
.equ	DUREE_POUR_VELCROTISER_H = 15

; booleen pour dire si on avance quand on a velcrotise
.equ	BOOL_POUSSE_VELCRO = 1
; duree pendant laquelle on avance quand on a velcro tis la planete
.equ	DUREE_POUSSE_VELCRO_L = 255
.equ	DUREE_POUSSE_VELCRO_H = 100

.equ	DUREE_ATTENDRE_L = 255
.equ	DUREE_ATTENDRE_H = 255

; booleen pour dire si on recule quand on a velcrotise
.equ	BOOL_TIRE_VELCRO = 1
; duree pendant laquelle on avance quand on a velcro tis la planete
.equ	DUREE_TIRE_VELCRO_L = 255
.equ	DUREE_TIRE_VELCRO_H = 150

;duree au bout de laquelle on monte le drapeau
.equ	TIME_MONTE_DRAPEAU = 60 ; secondes


;=====================
; Port mapping
;=====================
; Input 7 bits:
.equ	PORT_INPUT	= PORTD
.equ	DIR_INPUT	= DDRD
.equ	DETECT_SYNCRO	= 0x02 ; interup
.equ	DETECT_DEPART	= 0x01	
.equ	INTER_COLLISION = 0x00
.equ	INTER_BIEN_POSE = 0x03
.equ	INTER_PLAQUE_HA	= 0x04
.equ	INTER_PLAQUE_BA = 0x05
.equ	DETECT_DRAPEAU_OUT =0x06

; output 7 bits:
.equ	PORT_OUTPUT	= PORTB
.equ	DIR_OUTPUT	= DDRB
.equ	MOT_AVANCE_ON	= 0x01
.equ	MOT_AVANCE_DIR	= 0x02	
.equ	MOT_MONTE_ON	= 0x04
.equ	MOT_MONTE_DIR	= 0x08	; plaque descend =>dir=1
.equ	MOT_DRAPEAU_ON	= 0x10	
;.equ			= 0x20
.equ	LED_STATE	= 0x40	
.equ	LED_DRAPEAU	= 0x80	

;===========================
; TIMER, constante pour que 
; CK*1024*(255-timeD) = 0.039s
; pas de temps du compteur = 0.039s
; => octet de poids fort en seconde !
;===========================
.equ 	timeD		= 224
	
;========================
; variables
;========================
; Counters
.def	counter0 	= r23
.def	counter1 	= r24
.def	delaycounterL 	= r21
.def	delaycounterH 	= r22

.def  	timeFromBeginingL = r19
.def	timeFromBeginingH = r20

.def	outputreg = r25

.def	TmpA	=r16
.def	TmpB	=r17
.def	Tmp	=r18

;===========================
; 	EEPROM
;===========================
.eseg					;EEPROM segment
.org 0

;===========================
;	SOURCE CODE
;===========================	
.cseg					;CODE segment
.org $000
	rjmp __reset_
.org INT0addr
	rjmp __ext_int0_
.org OVF0addr
	rjmp __tim0_ovf_
.org ACIaddr
	reti
	
;========================
; program setup
;========================	
__reset_:
;definir la direction des ports
	ldi Tmp, 0xff
	out DIR_OUTPUT,Tmp
	ldi Tmp, 0x00
	out DIR_INPUT, Tmp
; set pull up for input
	ldi Tmp, 0xff
	out PORT_INPUT,Tmp
;couper tous les moteurs
	clr outputreg
	out PORT_OUTPUT, outputreg
;initialiser interruptions : disable all
	ldi Tmp,0x00
	out GIMSK, Tmp ; external int
	ldi Tmp,0x00
	out TIMSK, Tmp ; timer int
;enable interupts
	sei
;fin, commence le code
	rjmp start

;=========================
; interruption externe
;=========================
__ext_int0_ : 
	; tue l'interruption
	ldi Tmp,0x00
	out GIMSK, Tmp
	; c'est parti mon kiki !!!
	rjmp debut_match

;=========================
; interruption timer
;=========================
__tim0_ovf_ :
	; met a jour l'heure
time_plus_second :
	inc	timeFromBeginingL
	brcc	endtimeplus
	inc	timeFromBeginingH
endtimeplus:
	; reinitialise le compteur  la bonne valeur
	clr 	Tmp
	out 	TIFR, Tmp
	ldi	Tmp, timeD
	out 	TCNT0, Tmp
	reti
	
;=========================
;	delay
;=========================	
;delay : 
	mov	counter1, delaycounterH
delay1:	mov	counter0, delaycounterL
delay2:	dec	counter0
	brne	delay2
	dec	counter1
	brne	delay1
	ret	
	
;=========================
; mode idle
;=========================
start_idle_mode :
	;timer interupt disable
	ldi 	Tmp, 0x00
	out 	TIMSK, Tmp
	;external interrupt rising
	ldi 	Tmp, 0x23
	;external interrupt falling
	;ldi 	Tmp,0x22
	;external interrupt lowlevel
	;ldi 	Tmp,0x20
	out 	MCUCR, Tmp
	;external interrupt enable
	ldi 	Tmp, 0x40
	out 	GIMSK, Tmp
	sleep	; passe en idle mode, attend interruption externe	
	; suite dans __ext_int0_
loop1:
	rjmp loop1

;===============================
; initialisation de l'horologe
;===============================	
time_reset :
	ldi	timeFromBeginingL, 0x00
	ldi	timeFromBeginingH, 0x00
	; set period to 1 second
	ldi 	Tmp,0x05	; CK/1024
	out 	TCCR0, Tmp
	; initialise le compteur  la bonne valeur
	ldi	Tmp, timeD
	out 	TCNT0, Tmp
	clr 	Tmp
	out 	TIFR, Tmp
	;enable timer interrupt
	ldi 	Tmp,0x02
	out 	TIMSK, Tmp
	rjmp	wait_largue	
;=========================
; PROGRAM_PRINCIPAL
;=========================
start:	
	; on demarre, on economise les batteries
	; passer en mode sleep et attendre le dpart du match
	rjmp start_idle_mode
debut_match :
	; allume LED dpart
	sbr outputreg, LED_STATE ; set bit led 1
	out PORT_OUTPUT,outputreg
	;----- initialise l'horloge -------
	rjmp time_reset
	
	;----- Attend d'etre largue -------
wait_largue:
	sbis	PORT_INPUT, DETECT_DEPART
	rjmp 	wait_largue
	
	;----- on Demarre ------
	; on va vers l'avant
	sbr outputreg, MOT_AVANCE_DIR ; set bit
	out PORT_OUTPUT,outputreg
	sbr outputreg, MOT_AVANCE_ON ; set bit 
	out PORT_OUTPUT,outputreg
	
	;---- on attend la collision ou le timeout ------
wait_collision:
	sbis PORT_INPUT, INTER_COLLISION
	rjmp wait_collision
	
	;---- on arrete le moteur ------
	cbr outputreg, MOT_AVANCE_ON ; clear bit 
	out PORT_OUTPUT,outputreg
	
	;---- on velcrotise -------
	sbr outputreg, MOT_MONTE_DIR ; set bit
	out PORT_OUTPUT,outputreg
	sbr outputreg, MOT_MONTE_ON ; set bit 
	out PORT_OUTPUT,outputreg
;delay : 
	ldi	counter1, DUREE_POUR_VELCROTISER_H
delaya1:	
	ldi	counter0, DUREE_POUR_VELCROTISER_L
delaya2:	
	dec	counter0
	brne	delaya2
	dec	counter1
	brne	delaya1	
	cbr outputreg, MOT_MONTE_ON ; clear bit 
	out PORT_OUTPUT,outputreg
	
	;---- on souleve la planete -------
	cbr outputreg, MOT_MONTE_DIR ; clear bit
	out PORT_OUTPUT,outputreg
	sbr outputreg, MOT_MONTE_ON ; set bit 
	out PORT_OUTPUT,outputreg
wait_souleve:
	sbis PORT_INPUT, INTER_PLAQUE_HA
	rjmp wait_souleve
	cbr outputreg, MOT_MONTE_ON ; clear bit
	out PORT_OUTPUT,outputreg
	
	;---- on avance ------
	sbr outputreg, MOT_AVANCE_DIR ; set bit
	out PORT_OUTPUT,outputreg
	sbr outputreg, MOT_AVANCE_ON ; set bit 
	out PORT_OUTPUT,outputreg
;delay : 
	ldi	counter1, DUREE_POUSSE_VELCRO_H
delayb1:	
	ldi	counter0, DUREE_POUSSE_VELCRO_L
delayb2:	
	dec	counter0
	brne	delayb2
	dec	counter1
	brne	delayb1		
	cbr outputreg, MOT_AVANCE_ON ; clear bit 
	out PORT_OUTPUT,outputreg
	
	;---- on attend ------
;delay : 
	ldi	counter1, DUREE_ATTENDRE_H
delayc1:	
	ldi	counter0, DUREE_ATTENDRE_L
delayc2:	
	dec	counter0
	brne	delayc2
	dec	counter1
	brne	delayc1	
		
	;---- on recule ------
	cbr outputreg, MOT_AVANCE_DIR ; set bit
	out PORT_OUTPUT,outputreg
	sbr outputreg, MOT_AVANCE_ON ; set bit 
	out PORT_OUTPUT,outputreg
;delay : 
	ldi	counter1, DUREE_TIRE_VELCRO_H
delayd1:	
	ldi	counter0, DUREE_TIRE_VELCRO_L
delayd2:	
	dec	counter0
	brne	delayd2
	dec	counter1
	brne	delayd1		
	cbr outputreg, MOT_AVANCE_ON ; clear bit 
	out PORT_OUTPUT,outputreg
	
	;---- on monte sur la planete -----
	sbr outputreg, MOT_MONTE_DIR ; clear bit
	out PORT_OUTPUT,outputreg
	sbr outputreg, MOT_MONTE_ON ; set bit 
	out PORT_OUTPUT,outputreg
wait_onplanet:
	sbis PORT_INPUT, INTER_PLAQUE_BA
	rjmp wait_onplanet
	cbr outputreg, MOT_MONTE_ON ; clear bit
	out PORT_OUTPUT,outputreg
	
	;---- on attend que ce soit l'heure------
	ldi Tmp, TIME_MONTE_DRAPEAU
wait_drapeau:
	cp  timeFromBeginingH, Tmp
	brsh monte_drapeau
	rjmp wait_drapeau
	
	;---- regarde si bien en position drapeau -----
monte_drapeau :
	sbr outputreg, LED_DRAPEAU ; set bit 
	out PORT_OUTPUT,outputreg	
	; verifie qu'on est bien pose
wait_bienpose:
	sbis PORT_INPUT, INTER_BIEN_POSE
	rjmp wait_bienpose
	
	;---- monte drapeau -----
	sbr outputreg, MOT_DRAPEAU_ON ; set bit 
	out PORT_OUTPUT,outputreg	
wait_drapeau_haut:
	sbis PORT_INPUT, DETECT_DRAPEAU_OUT
	rjmp wait_drapeau_haut
	cbr outputreg, MOT_DRAPEAU_ON ; set bit 
	out PORT_OUTPUT,outputreg
	
	;---- on dit qu'on est content -----
fin :
	rjmp fin
