;==================================
; program for Velcro PMI
; robot ISIA 2001 - Space Odyssee
; www-isia.cma.fr/~robotm6/
; target = at90s1200, 8MHz
; compiled with avr studio from Atmel
;==================================

.include "1200def.inc"

;=====================
; Parametres reglbles
;=====================
; duree pendant laquelle on avance quand on a velcro tis la planete
.equ	MAX_TIME_AVANCE_L = 1
.equ	MAX_TIME_AVANCE_H = 2

.equ	DUREE_POUR_VELCROTISER_L = 255
.equ	DUREE_POUR_VELCROTISER_H = 15

; booleen pour dire si on avance quand on a velcrotise
.equ	BOOL_POUSSE_VELCRO = 1
; duree pendant laquelle on avance quand on a velcro tis la planete
.equ	DUREE_POUSSE_VELCRO_L = 255
.equ	DUREE_POUSSE_VELCRO_H = 100

.equ	DUREE_ATTENDRE_L = 255
.equ	DUREE_ATTENDRE_H = 255

; booleen pour dire si on recule quand on a velcrotise
.equ	BOOL_TIRE_VELCRO = 1
; duree pendant laquelle on avance quand on a velcro tis la planete
.equ	DUREE_TIRE_VELCRO_L = 255
.equ	DUREE_TIRE_VELCRO_H = 150

;duree au bout de laquelle on monte le drapeau
.equ	TIME_MONTE_DRAPEAU = 60 ; secondes


;=====================
; Port mapping
;=====================
; Input 7 bits:
.equ	PORT_INPUT	= PORTD
.equ	DIR_INPUT	= DDRD
.equ	DETECT_SYNCRO	= 0x02 ; interup
.equ	DETECT_DEPART	= 0x01	
.equ	INTER_COLLISION = 0x00
.equ	INTER_BIEN_POSE = 0x03
.equ	INTER_PLAQUE_HA	= 0x04
.equ	INTER_PLAQUE_BA = 0x05
.equ	DETECT_DRAPEAU_OUT =0x06

; output 7 bits:
.equ	PORT_OUTPUT	= PORTB
.equ	DIR_OUTPUT	= DDRB
.equ	MOT_AVANCE_ON	= 0x01
.equ	MOT_AVANCE_DIR	= 0x02	
.equ	MOT_MONTE_ON	= 0x04
.equ	MOT_MONTE_DIR	= 0x08	; plaque descend =>dir=1
.equ	MOT_DRAPEAU_ON	= 0x10	
;.equ			= 0x20
.equ	LED_STATE	= 0x40	
.equ	LED_DRAPEAU	= 0x80	

;===========================
; TIMER, constante pour que 
; CK*1024*(255-timeD) = 0.039s
; pas de temps du compteur = 0.039s
; => octet de poids fort en seconde !
;===========================
.equ 	timeD		= 224
	
;========================
; variables
;========================
; Counters
.def	counter0 	= r23
.def	counter1 	= r24
.def	delaycounterL 	= r21
.def	delaycounterH 	= r22

.def  	timeFromBeginingL = r19
.def	timeFromBeginingH = r20

.def	outputreg = r25

.def	TmpA	=r16
.def	TmpB	=r17
.def	Tmp	=r18

;===========================
; 	EEPROM
;===========================
.eseg					;EEPROM segment
.org 0

;===========================
;	SOURCE CODE
;===========================	
.cseg					;CODE segment
.org $000
	rjmp __reset_
.org INT0addr
	rjmp __ext_int0_
.org OVF0addr
	rjmp __tim0_ovf_
.org ACIaddr
	reti
	
;========================
; program setup
;========================	
__reset_:
;definir la direction des ports
	ldi Tmp, 0xff
	out DIR_OUTPUT,Tmp
	ldi Tmp, 0x00
	out DIR_INPUT, Tmp
;couper tous les moteurs
	clr outputreg
	out PORT_OUTPUT, outputreg
;initialiser interruptions : disable all
	ldi Tmp,0x00
	out GIMSK, Tmp
	ldi Tmp,0x00
	out TIMSK, Tmp
;enable interupts
	sei
;fin, commence le code
	rjmp run

;=========================
; interruption externe
;=========================
__ext_int0_ : 
	; tue l'interruption
	ldi Tmp,0x00
	out GIMSK, Tmp
	; c'est parti mon kiki !!!
	rjmp debut_match

;=========================
; interruption timer
;=========================
__tim0_ovf_ :
	; met a jour l'heure
time_plus_second :
	inc	timeFromBeginingL
	brne	endtimeplus
	inc	timeFromBeginingH
	inc 	outputreg
	out 	PORT_OUTPUT,outputreg
endtimeplus:
	; reinitialise le compteur  la bonne valeur
	clr 	Tmp
	out 	TIFR, Tmp
	ldi	Tmp, timeD
	out 	TCNT0, Tmp
	reti
	
run:	
	; on demarre, on economise les batteries
	; passer en mode sleep et attendre le dpart du match
	;rjmp start_idle_mode
debut_match :
	; allume LED dpart
	sbr outputreg, 0x00 ; set bit led 1
	out PORT_OUTPUT,outputreg
	;----- initialise l'horloge -------
time_reset :
	ldi	timeFromBeginingL, 0x00
	ldi	timeFromBeginingH, 0x00
	; set period to 1 second
	ldi 	Tmp,0x05	; CK/1024
	out 	TCCR0, Tmp
	; initialise le compteur  la bonne valeur
	ldi	Tmp, timeD
	out 	TCNT0, Tmp
	clr 	Tmp
	out 	TIFR, Tmp
	;enable timer interrupt
	ldi 	Tmp,0x02
	out 	TIMSK, Tmp

loop :
;	inc outputreg
;	out PORT_OUTPUT,outputreg
	rjmp loop