;==================================
; program for FLAG PMI
; robot ISIA 2001 - Space Odyssee
; www-isia.cma.fr/~robotm6/
; target = at90s1200, 8MHz
; compiled with avr studio from Atmel
;
;version de test sans ils et led de dpart
;==================================

.include "1200def.inc"

;=====================
; Parametres reglbles
;=====================
; choix de la strategie
.equ 	BOOL_POUSSE_PLANETE	= 1
.equ	BOOL_TIRE_PLANETE	= 1
.equ	BOOL_MONTE_DRAPEAU	= 1

; duree des taches (TIME)
; ou duree des delais d'attentes en cas de probleme (TIMEOUT)
.equ	TIMEOUT_AVANCE_1 	= 7 ; secondes
.equ	TIMEOUT_VELCROTISE 	= 60  ; secondes 
.equ	TIMEOUT_SOULEVE		= 10 ; secondes 
.equ	TIME_AVANCE_2 		= 10 ; secondes
.equ	TIME_WAIT_1		= 10 ; secondes ***
.equ	TIME_RECULE 		= 15 ; secondes
.equ	TIMEOUT_MONTE 		= 15 ; secondes
;duree au bout de laquelle on monte le drapeau
.equ	TIME_MONTE_DRAPEAU 	= 60 ; secondes ***
.equ	TIMEOUT_MONTE_DRAPEAU	= 15 ; secondes


;=====================
; Port mapping
;=====================
; Input 7 bits:
.equ	PORT_INPUT	= PORTD
.equ	PIN_INPUT	= PIND
.equ	DIR_INPUT	= DDRD
.equ	DETECT_SYNCRO	= 0x02 ; interup
.equ	ILS_DEPART	= 0x00	
.equ	INTER_COLLISION = 0x05
.equ	INTER_BIEN_POSE = 0x01
.equ	INTER_HAUT	= 0x03
.equ	INTER_BAS	= 0x04
.equ	DETECT_DRAPEAU_OUT =0x06 ; not connected

; output 8 bits:
.equ	PORT_OUTPUT	= PORTB
.equ	DIR_OUTPUT	= DDRB
.equ	MOT_AVANCE	= 0x02
.equ	MOT_RECULE	= 0x01	
.equ	MOT_MONTE	= 0x08
.equ	MOT_DESCEND	= 0x04	
.equ	MOT_DRAPEAU_ON	= 0x10	
;.equ			= 0x20
.equ	LED_STATE	= 0x40	
.equ	LED_DRAPEAU	= 0x80	


;===========================
; TIMER, constante pour que 
; CK*1024*(255-timeD) = 0.039s
; pas de temps du compteur = 0.039s
; => octet de poids fort en seconde !
;===========================
.equ 	timeD		= 224
	
;========================
; variables
;========================
; Counters
.def	counter0 	= r23
.def	counter1 	= r24
.def	delaycounterL 	= r21
.def	delaycounterH 	= r22

.def  	timeFromBeginingL = r19
.def	timeFromBeginingH = r20

.def	outputreg 	= r25
.def 	altitude 	= r26

.def	TmpTimer	=r16
.def	TmpB		=r17
.def	Tmp		=r18

;===========================
; 	EEPROM
;===========================
.eseg					;EEPROM segment
.org 0

;===========================
;	SOURCE CODE
;===========================	
.cseg					;CODE segment
.org $000
	rjmp __reset_
.org INT0addr
	rjmp __ext_int0_
.org OVF0addr
	rjmp __tim0_ovf_
.org ACIaddr
	reti
	
;========================
; program setup
;========================	
__reset_:
;definir la direction des ports
	ldi Tmp, 0xff
	out DIR_OUTPUT,Tmp
	ldi Tmp, 0x00
	out DIR_INPUT, Tmp
; set pull up for input
	ldi Tmp, 0xff
	out PORT_INPUT,Tmp
;couper tous les moteurs
	clr outputreg
	out PORT_OUTPUT, outputreg
;initialiser interruptions : disable all
	ldi Tmp,0x00
	out GIMSK, Tmp ; external int
	ldi Tmp,0x00
	out TIMSK, Tmp ; timer int
;disable interupts
	cli
;fin, commence le code
	rjmp start

;=========================
; interruption externe
;=========================
__ext_int0_ : 
	; tue l'interruption
	ldi Tmp,0x00
	out GIMSK, Tmp
	; c'est parti mon kiki !!!
	reti

;=========================
; interruption timer
;=========================
__tim0_ovf_ :
	; met a jour l'heure
time_plus_second :
	inc	timeFromBeginingL
	brne	endtimeplus
	inc	timeFromBeginingH
endtimeplus:
	; reinitialise le compteur  la bonne valeur
	sbrc	outputreg, 0x06
	rjmp	ledoff
	cbr	outputreg, LED_STATE
	rjmp	fintime
ledoff:
	sbr	outputreg, LED_STATE	
fintime:
	out 	PORT_OUTPUT,outputreg
	clr 	TmpTimer
	out 	TIFR, TmpTimer
	ldi	TmpTimer, timeD
	out 	TCNT0, TmpTimer
	reti

	
	
;=========================
;	delay
;=========================	
;delay : 
	mov	counter1, delaycounterH
delay1:	mov	counter0, delaycounterL
delay2:	dec	counter0
	brne	delay2
	dec	counter1
	brne	delay1
	ret	
	
;=========================
; mode idle
;=========================
start_idle_mode :
	;timer interupt disable
	ldi 	Tmp, 0x00
	out 	TIMSK, Tmp
	;external interrupt rising
	ldi 	Tmp, 0x23
	;external interrupt falling
	;ldi 	Tmp,0x22
	;external interrupt lowlevel
	;ldi 	Tmp,0x20
	out 	MCUCR, Tmp
	;external interrupt enable
	ldi 	Tmp, 0x40
	out 	GIMSK, Tmp
	sei
	sleep	; passe en idle mode, attend interruption externe
	rjmp	match_start ; au cas ou	
	; suite dans __ext_int0_
loop1:
	rjmp loop1

;===============================
; initialisation de l'horologe
;===============================	
time_reset :
	ldi	timeFromBeginingL, 0x00
	ldi	timeFromBeginingH, 0x00
	; set period to 1 second
	ldi 	TmpTimer,0x05	; CK/1024
	out 	TCCR0, TmpTimer
	; initialise le compteur  la bonne valeur
	ldi	TmpTimer, timeD
	out 	TCNT0, TmpTimer
	clr 	TmpTimer
	out 	TIFR, TmpTimer
	;enable timer interrupt
	ldi 	TmpTimer,0x02
	out 	TIMSK, TmpTimer
	sei
	rjmp	wait_largue	
;=========================
; PROGRAM_PRINCIPAL
;=========================
start:	

;******** DEBUG START *********
souleve2 :
	; stop moteurs
	cbr 	outputreg, MOT_AVANCE ; set bit
	cbr 	outputreg, MOT_MONTE ; set bit
	sbr 	outputreg, MOT_DESCEND; clear bit
	cbr 	outputreg, MOT_RECULE ; set bit
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend la collision ou le timeout ------
wait_soulevee2:
	sbic 	PIN_INPUT, INTER_HAUT
	rjmp	wait_soulevee2
	rjmp	real_match_start
;******** DEBUG END   *********

	; on demarre, on economise les batteries
	; passer en mode sleep et attendre le dpart du match
	rjmp 	start_idle_mode
	
match_start :
        ; check if it is really the begining, we must be under the ils
        sbis	PIN_INPUT, ILS_DEPART
        rjmp 	start
        
real_match_start:

;******** DEBUG START *********
	rjmp 	time_reset
wait_largue:
	cbr 	outputreg, MOT_AVANCE ; set bit
	cbr 	outputreg, MOT_MONTE ; set bit
	cbr 	outputreg, MOT_DESCEND; clear bit
	cbr 	outputreg, MOT_RECULE ; set bit
	out 	PORT_OUTPUT,outputreg
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,0x03
	add 	Tmp,timeFromBeginingH
wait_real_match_start:
	cp 	timeFromBeginingH, Tmp
	brsh 	avance_1
	rjmp	wait_real_match_start
	
;******** DEBUG END   *********

	; LED 0 (start) ON
	;sbr 	outputreg, LED_STATE ; set bit led 1
	;out 	PORT_OUTPUT,outputreg
	
	;----- timer reset -------
	rjmp 	time_reset
	
	;----- Attend d'etre largue -------
	;****** DEBUG
;wait_largue:
	rjmp	avance_1
	;***** DEBUG END
	sbic	PIN_INPUT, ILS_DEPART
	rjmp 	wait_largue
	
	;----- on Demarre ------
	; on va vers l'avant
avance_1:
	sbr 	outputreg, MOT_AVANCE ; set bit
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,TIMEOUT_AVANCE_1
	add 	Tmp,timeFromBeginingH
wait_collision:
	sbic 	PIN_INPUT, INTER_COLLISION
	rjmp	timeout_colli
	rjmp	velcrotise
timeout_colli:
	cp 	timeFromBeginingH, Tmp
	brsh 	pb_monte_sur_planet
	rjmp 	wait_collision	
	
	;------ on velcrotise la planete ------
velcrotise :
	; stop avance
	cbr 	outputreg, MOT_AVANCE ; set bit
	out 	PORT_OUTPUT,outputreg
	; regarde si on doit pousse la planete
	ldi	Tmp, BOOL_POUSSE_PLANETE
	cpi	Tmp, 0x00
	breq	tmp_monte_sur_planet
	; declenche montee
	sbr 	outputreg, MOT_MONTE ; clear bit
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,TIMEOUT_VELCROTISE
	add 	Tmp,timeFromBeginingH
wait_velcrotise:
	sbic 	PIN_INPUT, INTER_BAS
	rjmp	timeout_velcrotise_1
	rjmp	souleve
timeout_velcrotise_1:
	cp 	timeFromBeginingH, Tmp
	brsh 	souleve
	rjmp 	wait_velcrotise

;*************************************************	
	;---- on eu un probleme, on ne fait que monter sur la planete ------
pb_monte_sur_planet:
	; stop moteurs
	cbr 	outputreg, MOT_AVANCE ; set bit
	cbr 	outputreg, MOT_MONTE ; set bit
	cbr 	outputreg, MOT_RECULE ; set bit
	cbr 	outputreg, MOT_DESCEND ; set bit
	out 	PORT_OUTPUT,outputreg
	; declenche montee
	cbr 	outputreg, MOT_MONTE ; clear bit
	out 	PORT_OUTPUT,outputreg
	ldi 	Tmp,TIMEOUT_MONTE
	add 	Tmp,timeFromBeginingH
pb_wait_monte:
	sbic 	PIN_INPUT, INTER_BAS
	rjmp	pb_timeout_monte
	rjmp	tmp_fin
pb_timeout_monte:
	cp 	timeFromBeginingH,Tmp 
	brsh 	tmp_fin
	rjmp 	pb_wait_monte
;******************************************************

souleve :
	;*************** DEBUG
	rjmp tmp_fin
	;***************DEBUG END
	; stop moteurs
	cbr 	outputreg, MOT_AVANCE ; set bit
	cbr 	outputreg, MOT_MONTE ; set bit
	sbr 	outputreg, MOT_DESCEND; clear bit
	cbr 	outputreg, MOT_RECULE ; set bit
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend la collision ou le timeout ------
	ldi 	Tmp,TIMEOUT_SOULEVE
	add 	Tmp,timeFromBeginingH
wait_soulevee:
	sbic 	PIN_INPUT, INTER_HAUT
	rjmp	timeout_souleve_1
	rjmp	avance_2
timeout_souleve_1:
	cp 	timeFromBeginingH, Tmp
	brsh 	pb_monte_sur_planet
	rjmp 	wait_soulevee	

avance_2:	
	; stop moteurs ; declenche avance
	sbr 	outputreg, MOT_AVANCE; set bit
	cbr 	outputreg, MOT_MONTE ; set bit
	cbr 	outputreg, MOT_DESCEND ; set bit
	cbr 	outputreg, MOT_RECULE ; set bit
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend le timeout ------
	ldi 	Tmp,TIME_AVANCE_2
	add 	Tmp,timeFromBeginingH
wait_avance_2:
	cp 	timeFromBeginingH, Tmp
	brsh 	wait_1
	rjmp 	wait_avance_2	

; to avoid unreachable branch
tmp_fin:
	rjmp	fin
tmp_monte_sur_planet:
	rjmp	monte_sur_planet
		
wait_1 :
	; stop moteurs
	cbr 	outputreg, MOT_AVANCE
	cbr 	outputreg, MOT_MONTE 
	cbr 	outputreg, MOT_DESCEND 
	cbr 	outputreg, MOT_RECULE 
	out 	PORT_OUTPUT,outputreg
	; regarde si on doit pousse la planete
	ldi	Tmp, BOOL_TIRE_PLANETE
	cpi	Tmp, 0x00	
	breq	monte_sur_planet
	;---- on attend le timeout ------
	ldi 	Tmp,TIME_WAIT_1
	add 	Tmp,timeFromBeginingH
wait_wait_1:
	cp 	timeFromBeginingH, Tmp
	brsh 	recule
	rjmp 	wait_wait_1

recule :
	; stop moteurs et recule
	cbr 	outputreg, MOT_AVANCE
	cbr 	outputreg, MOT_MONTE 
	cbr 	outputreg, MOT_DESCEND 
	sbr 	outputreg, MOT_RECULE 
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend le timeout ------
	ldi 	Tmp,TIME_RECULE
	add 	Tmp,timeFromBeginingH
wait_recule:
	cp 	timeFromBeginingH, Tmp
	brsh 	monte_sur_planet
	rjmp 	wait_recule
	
monte_sur_planet:
	; stop moteurs et monte
	cbr 	outputreg, MOT_AVANCE
	sbr 	outputreg, MOT_MONTE 
	cbr 	outputreg, MOT_DESCEND 
	cbr 	outputreg, MOT_RECULE 
	ldi 	Tmp,TIMEOUT_MONTE
	add 	Tmp,timeFromBeginingH
wait_monte:
	sbic 	PIN_INPUT, INTER_BAS
	rjmp	timeout_monte_1
	rjmp	wait_time_monte_drapeau
timeout_monte_1:
	cp 	timeFromBeginingH,Tmp 
	brsh 	wait_time_monte_drapeau
	rjmp 	wait_monte	
	
wait_time_monte_drapeau :
	; stop moteurs
	cbr 	outputreg, MOT_AVANCE
	sbr 	outputreg, MOT_MONTE 
	cbr 	outputreg, MOT_DESCEND 
	cbr 	outputreg, MOT_RECULE 
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend le timeout ------
	ldi 	Tmp,TIME_MONTE_DRAPEAU
wait_time_monte_1:
	cp 	timeFromBeginingH, Tmp
	brsh 	monte_drapeau
	rjmp 	wait_time_monte_1
	
monte_drapeau:
	; stop moteurs
	cbr 	outputreg, MOT_AVANCE
	cbr 	outputreg, MOT_MONTE 
	cbr 	outputreg, MOT_DESCEND 
	cbr 	outputreg, MOT_RECULE 
	; alume led time drapeau
	sbr 	outputreg, LED_DRAPEAU ; set bit 
	out 	PORT_OUTPUT,outputreg
	
	; regarde si on doit monter le drapeau
	ldi	Tmp, BOOL_MONTE_DRAPEAU
	cpi	Tmp, 0x00	
	breq	fin
	
	; attend qu'on soit bien pose
wait_bien_pose:
	sbic 	PIN_INPUT, INTER_BIEN_POSE
	rjmp	wait_bien_pose
	
	; monte le drapeau
	sbr 	outputreg, LED_DRAPEAU ; set bit 
	sbr 	outputreg, MOT_DRAPEAU_ON ; set bit 
	out 	PORT_OUTPUT,outputreg
	
	;---- on attend le timeout ------
	ldi 	Tmp,TIMEOUT_MONTE_DRAPEAU
	add 	Tmp,timeFromBeginingH
wait_drapeau_haut:
	cp 	timeFromBeginingH, Tmp
	brsh 	fin
	rjmp 	wait_drapeau_haut
	
fin :	
	; stop moteurs
	cbr 	outputreg, MOT_AVANCE
	cbr 	outputreg, MOT_MONTE 
	cbr 	outputreg, MOT_DESCEND 
	cbr 	outputreg, MOT_RECULE 
	cbr 	outputreg, MOT_DRAPEAU_ON ; set bit 
	; etteind led state
	cbr 	outputreg, LED_STATE ; clear bit 
	out 	PORT_OUTPUT,outputreg	
start_powerdown_mode :
	;interupts disable
	cli
	;external interrupt rising & powerdown mode
	ldi 	Tmp, 0x33
	out 	MCUCR, Tmp
	sleep	; passe en powerdown mode, attend reset
	rjmp fin

