.include "1200def.inc"
;Input :
;-- [Add0,Add1,!Valid, Pos0,Pos1,Pos2,Pos3,Pos4]
;-- [ D0, D1, D2, D3, D4, D5 ,D6, B0]   

;output:
;-- B1, B2, B3, B4


;========================
; define constant
;========================
.equ	SHORT_DELAY = 12
.equ	ZERO_STEP_TIME = 30
.equ 	MINI_STEP_TIME = 200
.equ	LONG_DELAY = 0x7f

.equ	SERVO0_INIT = 0
.equ	SERVO1_INIT = 1
.equ	SERVO2_INIT = 15
.equ	SERVO3_INIT = 31
;========================
; define variable
;========================
.def 	Tmp	  = r20
.def	outputreg = r21
.def	inputreg  = r22
.def	delaya	  = r30
.def	delayb	  = r31

.def	v1	= r1
.def	v2	= r2
.def	v3	= r3
.def	v4	= r4

.def	re1	= r24
.def	re2	= r25
.def	re3	= r26
.def	re4	= r27

.def	b1	= r16
.def	b2	= r17
.def	b3	= r18
.def	b4	= r19

;========================
; interrupt vector table
;========================
.cseg	;CODE segment
.org $000 ; reset
	rjmp __reset_
.org INT0addr ; external interrupt
	rjmp __ext_int0_
.org OVF0addr ; timer overflow
	reti
.org ACIaddr ; analogic converter interrupt
	reti


;========================
; program init
;========================	
__reset_:
;define port direction
;input = D
	ldi Tmp, 0x00	
	out DDRD,Tmp
; set mode to pull-up -> PIND default value = 1, 
; +5V --[__ R=10KO __]-- * --|(C=4.7uF)|-- * --Gnd
;			 |	           |
;		 PIND -- * ---*/ inter *-- *
	ldi Tmp, 0xff
	out PORTD,Tmp
;output = B, except last pin
	ldi Tmp, 0xfe
	out DDRB, Tmp
	ldi outputreg, 0x01
	out PORTB,outputreg
;initialiser interruptions : disable all
	ldi Tmp,0x00
	out GIMSK, Tmp ; external int
	ldi Tmp,0x00
	out TIMSK, Tmp ; timer int
;enable interupts
	sei
;initialize servo data
	ldi Tmp, 0xff
	mov inputreg,Tmp
	ldi Tmp, SERVO0_INIT
	mov v1,Tmp
	ldi Tmp, SERVO1_INIT
	mov v2,Tmp
	ldi Tmp, SERVO2_INIT
	mov v3,Tmp
	ldi Tmp, SERVO3_INIT
	mov v4,Tmp
	
;fin, commence le code
	rjmp read

;====================
; external interrupt
;====================
__ext_int0_:	; get a new request, we store the new parameters
	in inputreg, PIND ; 1
	sbic PINB,0x01 ; 0.5
	sbr inputreg,0x07;1
	ldi Tmp, 0x0a
	cp delaya,Tmp ;1
	brlo interuptend ;0.5
	sbci delaya, 0x09 ;1
	reti ;4
interuptend : 
	ldi delaya, 0x01 ;1
	reti ;4 

;=====================
; analyse the request
;=====================
;-- 2 first bit = servo number --
analyse:
	sbrs inputreg,0x00
	rjmp bitsx0
bitsx1:	sbrs inputreg,0x01
	rjmp bits01
bits11:	andi inputreg,0xf8
	mov  v4,inputreg
	lsr v4
	lsr v4
	lsr v4
	rjmp analyseend
bitsx0:
	sbrs inputreg,0x01
	rjmp bits00
bits10:
	andi inputreg,0xf8
	mov  v3,inputreg
	lsr v3
	lsr v3
	lsr v3
	rjmp analyseend
	
bits01:	andi inputreg,0xf8
	mov  v2,inputreg
	lsr v2
	lsr v2
	lsr v2
	rjmp analyseend
bits00:
	andi inputreg,0xf8
	mov  v1,inputreg
	lsr v1
	lsr v1
	lsr v1
	rjmp analyseend


analyseend:
	rjmp sort
	
	
;===============================
; sort times by greater order
;===============================
sort :
	cp v1,v2
	brlo sort12
sort21:	mov re1,v2 ; 2<1
	mov re2,v1
	ldi b1,0x04
	ldi b2,0x02
	rjmp sortx3
sort12: mov re1,v1 ; 1<2
	mov re2,v2
	ldi b1,0x02
	ldi b2,0x04
sortx3 :
	cp v3,re1
	brlo sort31
	cp v3,re2
	brlo sort32
	mov re3,v3	; 1<2<3
	ldi b3,0x08
	rjmp sortx4
sort32: mov re3,re2	; 1<3<2
	mov re2,v3
	mov b3,b2
	ldi b2,0x08
	rjmp sortx4
sort31: mov re3,re2	; 3<1<2
	mov re2,re1
	mov re1,v3
	mov b3,b2
	mov b2,b1
	ldi b1,0x08

sortx4 :
	cp v4,re1
	brlo sort41
	cp v4,re2
	brlo sort42
	cp v4,re3
	brlo sort43
	mov re4,v4	; 1<2<3<4
	ldi b4,0x10
	rjmp sortend
sort43: mov re4,re3	; 1<2<4<3
	mov re3,v4
	mov b4,b3
	ldi b3,0x10
	rjmp sortend
sort42: mov re4,re3	; 1<4<2<3
	mov re3,r2
	mov re2,v4
	mov b4,b3
	mov b3,b2
	ldi b2,0x10
	rjmp sortend
sort41: mov re4,re3	; 4<1<2<3
	mov re3,re2
	mov re2,re1
	mov re1,v4
	mov b4,b3
	mov b3,b2
	mov b2,b1
	ldi b1,0x10
sortend :	
	com b1
	com b2
	com b3
	com b4
; compute DT between servo orders
	sub re4,re3
	sub re3,re2
	sub re2,re1

	rjmp timing	
 
;===============================
; set motor on
;===============================
timing :
	; set servo ON
	ori outputreg, 0x1e 
	out PORTB,outputreg
	
	ldi delayb,SHORT_DELAY
delayi1:ldi delaya,ZERO_STEP_TIME
delayi2:dec delaya
	brne delayi2
	dec delayb 
	brne delayi1
	
servo1:	cpi re1, 0x00
	breq servo2
	mov delayb,re1
delayz1:ldi delaya,MINI_STEP_TIME
delayz2:dec delaya
	brne delayz2
	dec delayb 
	brne delayz1 
	
servo2:	and outputreg,b1
	out PORTB, outputreg
	cpi re2, 0x00
	breq servo3
	mov delayb,re2
delaye1:ldi delaya,MINI_STEP_TIME
delaye2:dec delaya
	brne delaye2
	dec delayb 
	brne delaye1
	 
servo3:	and outputreg,b2
	out PORTB, outputreg
	cpi re3, 0x00
	breq servo4
	mov delayb,re3
delayr1:ldi delaya,MINI_STEP_TIME
delayr2:dec delaya
	brne delayr2
	dec delayb 
	brne delayr1
	
servo4:	and outputreg,b3
	out PORTB, outputreg 
	cpi re4, 0x00
	breq endservo
	mov delayb,re4
delayt1:ldi delaya,MINI_STEP_TIME
delayt2:dec delaya
	brne delayt2
	dec delayb 
	brne delayt1
	
endservo:
	and outputreg,b4
	out PORTB, outputreg

;============================
; read new request
;============================
read :
        sbic PIND,0x02 ; if D2=1, data invalid (due to i2c init)
        rjmp wait
	in inputreg, PIND ; 1
	sbic PINB,0x00 ; 0.5 ; read the first bit on port D
	sbr inputreg,0x80 ;1
	rjmp wait
	
;============================
; wait around 15ms
;============================
wait :
	ldi delayb,LONG_DELAY ;1
delayl1:ldi delaya,0xa6 ;1
delayl2:nop		;1
	dec delaya      ;1
	brne delayl2    ;0.5
	dec delayb      ;1
	brne delayl1    ;1
	
	rjmp analyse