;==================================
; program to test the clock
; robot ISIA 2001 - Space Odyssee
; www-isia.cma.fr/~robotm6/
; target = at90s1200, 8MHz
; compiled with avr studio from Atmel
;==================================

.include "1200def.inc"

;=====================
; Port mapping
;=====================
.equ	PORT_INPUT	= PORTD
.equ	DIR_INPUT	= DDRD

.equ	PORT_OUTPUT	= PORTB
.equ	DIR_OUTPUT	= DDRB

;===========================
; TIMER, constant so that 
; CK*1024*(255-timeD) = 0.039s
; step of the interrupt = 0.039s
; => *256 = 1 second !
;===========================
.equ 	timeD		= 224
	
;========================
; variables
;========================
.def  	timeFromBeginingL = r19
.def	timeFromBeginingH = r20

.def	outputreg = r25

.def	Tmp	=r18

;===========================
; 	EEPROM
;===========================
.eseg					;EEPROM segment
.org 0

;===========================
;	SOURCE CODE
;===========================	
.cseg					;CODE segment
.org $000
	rjmp __reset_
.org INT0addr
	reti
.org OVF0addr
	rjmp __tim0_ovf_
.org ACIaddr
	reti
	
;========================
; program setup
;========================	
__reset_:
;definir la direction des ports
	ldi Tmp, 0xff
	out DIR_OUTPUT,Tmp
	ldi Tmp, 0x00
	out DIR_INPUT, Tmp
;initialiser interruptions : disable all
	ldi Tmp,0x00
	out GIMSK, Tmp
	ldi Tmp,0x00
	out TIMSK, Tmp
;enable interupts
	sei
;fin, commence le code
	rjmp main

;=========================
; timer interrupt 
;=========================
__tim0_ovf_ :
	; met a jour l'heure
time_plus_second :
	inc	timeFromBeginingL
	brne	end_time_plus
	; increment seconds
	inc	timeFromBeginingH
	inc 	outputreg
	out 	PORT_OUTPUT,outputreg
end_time_plus:
	; reset timer
	clr 	Tmp
	out 	TIFR, Tmp
	ldi	Tmp, timeD
	out 	TCNT0, Tmp
	reti
	
	
;===========================
; main program
;===========================
main:	
	;----- clear all outputs ------
	clr outputreg
	out PORT_OUTPUT,outputreg
	
	;----- timer_counter reset -------
time_reset :
	; clear register
	clr	timeFromBeginingL
	clr	timeFromBeginingH
	; set period to 1 second
	ldi 	Tmp,0x05	; CK/1024
	out 	TCCR0, Tmp
	; initialise timer to the right value so that 256*timerinterupt = 1second
	ldi	Tmp, timeD
	out 	TCNT0, Tmp
	clr 	Tmp
	out 	TIFR, Tmp
	;enable timer interrupt
	ldi 	Tmp,0x02
	out 	TIMSK, Tmp

	;---- wait forever ------
loop :
	rjmp loop